@@ -18,19 +18,32 @@ test_that("a subprocess can be spawned and killed", {
   
   expect_true(process_exists(handle))
   
+  # we need to clean-up 'manually'
+  process_write(handle, "cat(tempdir())\n")
+  path <- process_read(handle, PIPE_STDOUT, TIMEOUT_INFINITE)
+  expect_true(dir.exists(path))
+  on.exit(unlink(path, TRUE, TRUE), add = TRUE)
+
   process_kill(handle)
   expect_equal(process_wait(handle, TIMEOUT_INFINITE), killed_exit_code)
   expect_equal(process_state(handle), "terminated")
-  expect_false(process_exists(handle))
+  expect_true(wait_until_exits(handle))
 })
 
 
 test_that("waiting for a child to exit", {
-  on.exit(process_kill(handle))
+  on.exit(process_terminate(handle))
   handle <- R_child()
 
   process_wait(handle, TIMEOUT_IMMEDIATE)
   expect_equal(process_state(handle), "running")
+  
+  # we need to clean-up 'manually'
+  process_write(handle, "cat(tempdir())\n")
+  path <- process_read(handle, PIPE_STDOUT, TIMEOUT_INFINITE)
+  expect_true(dir.exists(path))
+  on.exit(unlink(path, TRUE, TRUE), add = TRUE)
+  
   process_kill(handle)
 
   expect_equal(process_wait(handle, TIMEOUT_INFINITE), killed_exit_code)
@@ -58,7 +71,7 @@ test_that("can expand paths", {
 
 
 test_that("handle can be printed", {
-  on.exit(process_kill(handle))
+  on.exit(terminate_gracefully(handle))
   handle <- R_child()
   
   path <- gsub("\\\\", "\\\\\\\\", normalizePath(R_binary()))
